import httpx

def get(url: str, params: dict = None, verify: bool = True):
    response = httpx.get(url, verify=verify, timeout=10.0) if not params else httpx.get(url, params=params, timeout=10.0)

    if response.status_code == httpx.codes.OK:
        return response.content
    raise ValueError(f"We could not connect to the page {url}. Status Code: {response.status_code}")


def _bs_to_dollar(value: float, price_monitor: float):
    return value / price_monitor

def _dollar_to_bs(value: float, price_monitor: float):
    return value * price_monitor

def _isnumber(_object):
    if isinstance(_object, int) or isinstance(_object, float):
        return True
    return False

def currency_converter(type: str, value, monitor: dict):
    """
    Convierte una cantidad de dinero de una moneda a otra utilizando los datos de un monitor específico.
    """
    price_monitor = monitor['price']

    if _isnumber(value):
        if type == 'VES':
            return _bs_to_dollar(value, float(price_monitor))
        elif type == 'USD':
            return _dollar_to_bs(value, float(price_monitor))
        else:
            raise ValueError(f"The type must be USD or VES not {type}")
    else:
        raise TypeError(f"The value must be an integer or floating point number, not {value}")


from pyDolarVenezuela import network
from pyDolarVenezuela.tools import TimeDollar

import json

def _convert_specific_format(text: str, character: str = '_') -> str:
    acentos = {'á': 'a', 'é': 'e', 'í': 'i', 'ó': 'o', 'ú': 'u'}
    for acento, sin_acento in acentos.items():
        text = text.lower().replace(acento, sin_acento).replace(' ', character)
    return text

def _convert_dollar_name_to_monitor_name(monitor_name: str):
    if monitor_name.split(' ')[0] == 'Dólar' and monitor_name != 'Dólar Today':
        if monitor_name == 'Dólar Monitor':
            return 'EnParaleloVzla'
        else:
            return monitor_name.split(' ')[1]
    return monitor_name

class CriptoDolar:
    def __init__(self, url: str) -> None:
        response           = network.get(url + "coins/latest")
        self.json_response = json.loads(response)
    
    def _load(self):
        t = TimeDollar()
        self.data = {}

        for monitor in self.json_response:
            if 'bolivar' in monitor['type'] or 'bancove' in monitor['type']:
                data = {
                    'title': _convert_dollar_name_to_monitor_name(monitor['name']),
                    'price': monitor['price'],
                    'price_old': monitor['priceOld'],
                    'type': 'bank' if monitor['type'] == 'bancove' else 'monitor',
                    'last_update': t.get_time(monitor['updatedAt']),
                }

                self.data[_convert_specific_format(data['title'])] = data
    
    def get_values(self, monitor_code: str = None, name_property: str = None, prettify: bool = True):
        self._load()

        if not monitor_code:
            return self.data
        
        try:
            monitor_data = self.data[monitor_code.lower()]
            if name_property:
                value = monitor_data[name_property]
                return f'Bs. {value}' if prettify and name_property == 'price' else value
            return monitor_data
        except KeyError:
            raise KeyError("Does not match any of the properties that were provided in the dictionary. Most information: https://github.com/fcoagz/pyDolarVenezuela")


from pyDolarVenezuela import pages
from pyDolarVenezuela import network
from pyDolarVenezuela.tools import currency_converter
from .provider import Provider

import json
from colorama import Fore

version = '1.3.4'

def check_dependence_version():
    response = network.get("https://pypi.org/pypi/pydolarvenezuela/json")
    latest_version = json.loads(response)["info"]["version"]

    if version != latest_version:
        print(f"{Fore.GREEN}New version: {latest_version}.{Fore.RESET} {Fore.RED}Current version {version}.{Fore.RESET} write the following command: pip install --upgrade pyDolarVenezuela\n")

check_dependence_version()

def getdate():
    from pyDolarVenezuela.tools import TimeDollar
    t = TimeDollar()

    return t.get_time_zone()

class Monitor:
    def __init__(self, provider: pages.Monitor, ) -> None:
        """
        La clase Monitor permite consultar los precios del dólar en diversos monitores en Venezuela. \n
        El método `get_value_monitors` permite acceder a los datos almacenados en el diccionario.
        """
        if not isinstance(provider, pages.Monitor):
            raise TypeError("El parámetro debe ser un objeto del tipo Monitor.")
        
        self.provider = provider

    def get_value_monitors(self, monitor_code: str = None, name_property: str = None, prettify: bool = False):
        """
        El parámetro `monitor_code` indica el código del monitor del cual se desea obtener información, \
        mientras que el parámetro `prettify` permite mostrar los precios en formato de moneda con el símbolo de Bolívares. \
        Si se proporciona un nombre de propiedad válido, se devolverá el valor correspondiente para ese monitor.
        """
        return Provider.select_monitor(self.provider, monitor_code, name_property, prettify)

__all__ = ['pages', 'currency_converter', 'getdate', 'Monitor']


from pyDolarVenezuela import network
from bs4 import BeautifulSoup

def _get_price_and_symbol_in_simple(strong_tag: BeautifulSoup):
    price = str(strong_tag.find_next('strong').text).split('Bs.')[-1][:-1].strip()
    symbol = str(strong_tag.find_next('strong').text).split('Bs.')[-1][-1]

    return price, symbol

def _get_values_monitors(url: str):
    response = network.get(url)
    soup = BeautifulSoup(response, "html.parser")

    section_information_dollar = soup.find('div', 'td-post-content')
    monitors_simple = section_information_dollar.find('p', style='font-size:16px')
    image_monitors = section_information_dollar.find_all('div', 'wp-block-image')

    information  = _extract_simple_data(monitors_simple)
    image = _extract_image_data(image_monitors)
    result = {}

    for key, value in information.items():
        result[key] = value
        if key in image:
            result[key]['image'] = image[key]

    return result

def _extract_simple_data(monitors_simple):
    result = {}
    for strong_tag in monitors_simple.find_all('strong'):
        title_map = {
            'Dólar Oficial BCV': 'BCV',
            'Monitor': 'EnParaleloVzla',
            'Dolartoday': 'DolarToday'
        }
        for key, value in title_map.items():
            if key in strong_tag.text:
                price, symbol = _get_price_and_symbol_in_simple(strong_tag)
                result[value.lower()] = {'title': value, 'price': price.replace(',', '.'), 'symbol': symbol}
    return result

def _extract_image_data(image_monitors):
    result = {}
    for image in image_monitors:
        title_map = {
            'Tasa': 'bcv',
            'Monitor': 'enparalelovzla',
            'Dolartoday': 'dolartoday'
        }
        alt_text = str(image.find('img')['alt']).split(' ')[0]
        for key, value in title_map.items():
            if key in alt_text:
                if value not in result:
                    result[value] = [image.find('img')['src']]
                else:
                    result[value].append(image.find('img')['src'])
    return result

class iVenezuela:
    def __init__(self, url: str) -> None:
        response  = network.get(url + 'precio-dolar-venezuela-dolartoday-monitor-oficial-bcv-hoy-tipo-de-cambio/')
        self.soup = BeautifulSoup(response, "html.parser")
    
    def _load(self):
        section_information_dollar = self.soup.find('div', 'td-post-content')

        for page in section_information_dollar.find_all('p', 'has-text-align-center'):
            if page.find('a'):
                self.data = _get_values_monitors(page.find('strong').find('a')['href'])
                self.data['date'] = ' '.join(page.find('a').text.split('hoy  ')[1::])
    
    def get_values(self, monitor_code: str = None, name_property: str = None, prettify: bool = False):
        self._load()

        if not monitor_code:
            return self.data
        
        try:
            monitor_data = self.data[monitor_code.lower()]
            if name_property:
                value = monitor_data[name_property]
                return value
            return monitor_data
        except KeyError:
            raise KeyError("Does not match any of the properties that were provided in the dictionary. Most information: https://github.com/fcoagz/pyDolarVenezuela")


from pyDolarVenezuela import network

import json

monitors = ["dolar-em", "monitor_dolar_venezuela", "enparalelovzla", "monitor_dolar_vzla", "petro",
"bcv", "remesas_zoom", "italcambio","bancamiga","banco_de_venezuela","banco-exterior",
"banplus","bnc","banesco","bbva_provincial","mercantil","otras_instituciones",
"binance","airtm","reserve","syklo","yadio",
"dolartoday","mkambio" ,"cambios-r&a" ,"paypal" ,"zinli" ,
"skrill" ,"amazon_gift_card"]

class ExchangeMonitor:
    def __init__(self, url: str) -> None:
        self.url = url + "ajax/widget-unique"
    
    def get_values(self, monitor_code: str = None, name_property: str = None, prettify: bool = True):
        results = []

        for monitor in monitors:
            if not monitor_code or monitor.lower() == monitor_code.lower():
                params   = {'country': 've', 'type': monitor.replace('_', '-')}
                response = network.get(self.url, params=params)
                json_response = json.loads(response)

                data = {
                    'title': json_response['name'],
                    'price': str(json_response['price']).replace(',', '.'),
                    'symbol': json_response['symbol'],
                    'change': json_response['change'],
                    'percent': json_response['percent']
                }

                if name_property:
                    value = data.get(name_property)
                    return f'Bs. {value}' if prettify and name_property == 'price' else value

                results.append(data)

        return results if len(results) > 1 else results[0]


from datetime import datetime 
from babel.dates import format_date, format_time, get_timezone

class TimeDollar(object):
    def __init__(self) -> None:
        self.locality = get_timezone('America/Caracas')
        self.datetime_obj = datetime.now(self.locality)
    
    def get_time(self, date: str, dateofApi: bool = False):
        if dateofApi:
            datetime_obj = datetime.strptime(date, '%Y-%m-%d %H:%M')
        else:
            datetime_obj = datetime.strptime(date, '%Y-%m-%dT%H:%M:%S.%fZ')
        
        return datetime_obj.strftime("%d/%m/%Y, %I:%M %p")
    
    def get_time_zone(self):
        formatted_date = format_date(self.datetime_obj.date(), format='full', locale='es')
        formatted_time = format_time(self.datetime_obj.time(), format='h:mm:ss a', locale='es')

        return {
            "date": formatted_date,
            "time": formatted_time
        }


import requests
from bs4 import BeautifulSoup

requests.packages.urllib3.disable_warnings()

def _get_rate_by_id(tag_id: str, soup: BeautifulSoup):
    return soup.find(id=tag_id).find("strong").text.strip().replace(',', '.')

def _get_time(soup: BeautifulSoup):
    date = soup.find("span", "date-display-single")
    return [date.text.strip().replace('  ', ' '), date.get("content").split('T')[0]]

class BCV:
    def __init__(self, url: str) -> None:
        response  = requests.get(url, verify=False)
        response.raise_for_status()
        self.soup = BeautifulSoup(response.content, "html.parser")
    
    def _load(self) -> None:
        section_tipo_de_cambio_oficial = self.soup.find("div", "view-tipo-de-cambio-oficial-del-bcv")

        self.rates = {
            "eur": {"currency": "Euro", "price": _get_rate_by_id("euro", section_tipo_de_cambio_oficial)},
            "cny": {"currency": "Yuan chino", "price": _get_rate_by_id("yuan", section_tipo_de_cambio_oficial)},
            "try": {"currency": "Lira turca", "price": _get_rate_by_id("lira", section_tipo_de_cambio_oficial)},
            "rub": {"currency": "Rublo ruso", "price": _get_rate_by_id("rublo", section_tipo_de_cambio_oficial)},
            "usd": {"currency": "Dólar estadounidense", "price": _get_rate_by_id("dolar", section_tipo_de_cambio_oficial)},
            "date": _get_time(section_tipo_de_cambio_oficial)
        }

    def get_values(self, currency_code: str = None, name_property: str = None, prettify: bool = True):
        self._load()

        if not currency_code:
            return self.rates
        
        try:
            monitor_data = self.rates[currency_code.lower()]
            if name_property:
                value = monitor_data[name_property]
                return f'Bs. {value}' if prettify and name_property == 'price' else value
            return monitor_data
        except KeyError:
            raise KeyError("Does not match any of the properties that were provided in the dictionary. Most information: https://github.com/fcoagz/pyDolarVenezuela")


from pyDolarVenezuela.pages import Monitor

from .bcv import BCV
from .criptodolar import CriptoDolar
from .exchangemonitor import ExchangeMonitor
from .ivenezuela import iVenezuela
from .dpedidos import Dpedidos

class Provider:
    @classmethod
    def select_monitor(self, monitor: Monitor, monitor_code: str = None, name_property: str = None, prettify: bool = False):
        page = monitor.name

        if page == 'Banco Central de Venezuela':
            return BCV(monitor.provider).get_values(monitor_code, name_property, prettify)
        elif page == 'Exchange Monitor':
            return ExchangeMonitor(monitor.provider).get_values(monitor_code, name_property, prettify)
        elif page == 'Cripto Dolar':
            return CriptoDolar(monitor.provider).get_values(monitor_code, name_property, prettify)
        elif page == 'iVenezuela':
            return iVenezuela(monitor.provider).get_values(monitor_code, name_property, prettify)
        elif page == 'Monitor Dolar Venezuela':
            return Dpedidos(monitor.provider).get_values(monitor_code, name_property, prettify)


from pyDolarVenezuela import network
from pyDolarVenezuela.tools import TimeDollar

import json

monitors = {'binance': 'Binance', 'dolartoday': 'DolarToday', 'yadio': 'Yadio', 'airtm': 'Airtm', 'cambiosrya': 'Cambios R&A', 'mkambio': 'Mkambio', 'bcv': 'BCV', 'promediovip': 'EnParaleloVzlaVip', 'prom_epv': 'EnParalelovzla'}

class Dpedidos:
    def __init__(self, url: str) -> None:
        response           = network.get(url + 'minmaxhistorial')
        json_response = json.loads(response)

        self.json_response = json_response['result'][0]
    
    def get_values(self, monitor_code: str = None, name_property: str = None, prettify: bool = True):
        t = TimeDollar()
        result = {}

        for key, title in monitors.items():
            if key in self.json_response and self.json_response[key] != '0':
                data = {
                    'title': title,
                    'price': round(float(self.json_response[key]), 2)
                }

                if key == 'prom_epv':
                    data['last_update'] = t.get_time(f"{self.json_response['fecha_epv']} {'13' if self.json_response['hora_epv'] == '1' else self.json_response['hora_epv']}:00", True)
                
                result[title.lower()] = data
        result['last_update'] = t.get_time(f"{self.json_response['fecha']} {self.json_response['hora']}", True)

        if not monitor_code:
            return result
        
        try:
            monitor_data = result[monitor_code.lower()]
            if name_property:
                value = monitor_data[name_property]
                return value
            return monitor_data
        except KeyError:
            raise KeyError("Does not match any of the properties that were provided in the dictionary. Most information: https://github.com/fcoagz/pyDolarVenezuela")


from dataclasses import dataclass

@dataclass
class Monitor:
    name: str
    provider: str

BCV = Monitor(
    name="Banco Central de Venezuela",
    provider="http://www.bcv.org.ve/"
)

CriptoDolar = Monitor(
    name="Cripto Dolar",
    provider="https://exchange.vcoud.com/"
)

ExchangeMonitor = Monitor(
    name="Exchange Monitor",
    provider="https://exchangemonitor.net/"
)

iVenezuela = Monitor(
    name="iVenezuela",
    provider="https://www.ivenezuela.travel/"
)

Dpedidos = Monitor(
    name="Monitor Dolar Venezuela",
    provider="https://api.lyldesarrollo.com/"
)